package eu.pozimski.alarmclockconfigurator

import android.os.Build
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.annotation.RequiresApi
import androidx.compose.foundation.*
import androidx.compose.foundation.layout.*
import androidx.compose.material.*
import androidx.compose.runtime.*
import androidx.compose.ui.platform.LocalContext
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import eu.pozimski.alarmclockconfigurator.http.WakeupClient
import eu.pozimski.alarmclockconfigurator.http.WakeupTimeCache
import eu.pozimski.alarmclockconfigurator.ui.elements.*
import eu.pozimski.alarmclockconfigurator.ui.theme.AlarmClockConfiguratorTheme
import eu.pozimski.alarmclockconfigurator.viewmodels.ConfigurationViewModel
import io.ktor.client.*
import io.ktor.client.engine.android.*
import io.ktor.client.plugins.contentnegotiation.*
import io.ktor.serialization.kotlinx.json.*
import io.ktor.util.collections.*
import io.ktor.util.date.*
import java.util.*

class MainActivity : ComponentActivity() {

    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val config = AndroidEngineConfig()
        config.threadsCount = 1
        config.socketTimeout = 10_000
        config.connectTimeout = 10_000

        setContent {
            AlarmClockConfiguratorTheme {
                Surface(
                    color = MaterialTheme.colors.background
                ) {

                    val context = LocalContext.current
                    val configurationViewModel by remember {
                        mutableStateOf(ConfigurationViewModel(context))
                    }
                    val apiUrl = configurationViewModel.apiUrl.collectAsState()

                    if (apiUrl.value != "") {
                        val wakeupClient by remember {
                            mutableStateOf(WakeupClient(HttpClient(AndroidClientEngine(config)) {
                                install(
                                    ContentNegotiation
                                ) {
                                    json()
                                }
                            }, apiUrl.value))
                        }
                        MainView(wakeupClient, configurationViewModel)
                    } else {
                        URLPrompt(configurationViewModel)
                    }


                }
            }
        }
    }
}

@RequiresApi(Build.VERSION_CODES.TIRAMISU)
@Composable
fun MainView(wakeupClient: WakeupClient, configurationViewModel: ConfigurationViewModel) {
    val navController = rememberNavController()
    val localDateCache = remember {
        WakeupTimeCache.forLocalDate(wakeupClient)
    }
    val weekDayCache = remember {
        WakeupTimeCache.forDayOfWeek(wakeupClient)
    }
    Column {
        Row {
            NavigationBar(navController = navController)
        }
        Row {

            NavHost(navController = navController, startDestination = "calendar") {
                composable("calendar") {
                    CalendarView(localDateCache)
                }

                composable("weekdays") {
                    WeekDayView(weekDayCache)
                }
                composable("settings") { Settings(configurationViewModel) }
            }
        }
    }
}