package eu.pozimski.alarmclockconfigurator.http

import io.ktor.util.collections.*
import kotlinx.datetime.LocalDate
import java.time.DayOfWeek

class WakeupTimeCache<T>(
    private val wakeupClient: WakeupClient,
    private val retrievalFunction: suspend (WakeupClient, T) -> WakeupTime?,
    private val deletionFunction: suspend (WakeupClient, T) -> Unit,
    private val deactivationFunction: suspend (WakeupClient, T) -> Unit,
    private val writeFunction: suspend (WakeupClient, T, WakeupTime) -> Unit
) {
    private val wakeupTimeMap: MutableMap<T, Pair<WakeupTime?, Boolean>> = ConcurrentMap()

    companion object {
        fun forDayOfWeek(
            wakeupClient: WakeupClient
        ): WakeupTimeCache<DayOfWeek> {
            return WakeupTimeCache(wakeupClient,
                { client: WakeupClient, key: DayOfWeek -> client.fetchWakeup(key) },
                { client: WakeupClient, key: DayOfWeek -> client.deleteWakeup(key) },
                { client: WakeupClient, key: DayOfWeek -> client.deactivateWakeup(key) }
            ) { client: WakeupClient, day: DayOfWeek, wakeupTime: WakeupTime ->
                client.writeWakeup(
                    day,
                    wakeupTime
                )
            }
        }

        fun forLocalDate(
            wakeupClient: WakeupClient
        ): WakeupTimeCache<LocalDate> {
            return WakeupTimeCache(wakeupClient,
                { client: WakeupClient, key: LocalDate -> client.fetchWakeup(key) },
                { client: WakeupClient, key: LocalDate -> client.deleteWakeup(key) },
                { client: WakeupClient, key: LocalDate -> client.deactivateWakeup(key) }
            ) { client: WakeupClient, day: LocalDate, wakeupTime: WakeupTime ->
                client.writeWakeup(
                    day,
                    wakeupTime
                )
            }
        }
    }

    suspend fun get(key: T): WakeupTime? {
        if (wakeupTimeMap.containsKey(key)) {
            return wakeupTimeMap[key]!!.first
        }
        val wakeupTime = retrievalFunction.invoke(wakeupClient, key)
        if (wakeupTime == null) {
            wakeupTimeMap[key] = Pair(null, false)
        }
        return wakeupTime
    }

    suspend fun put(key: T, wakeupTime: WakeupTime) {
        wakeupTimeMap[key] = Pair(wakeupTime, true)
        writeFunction.invoke(wakeupClient, key, wakeupTime)
    }

    suspend fun delete(key: T) {
        wakeupTimeMap.remove(key)
        deletionFunction.invoke(wakeupClient, key)
    }

    suspend fun deactivate(key: T) {
        wakeupTimeMap[key] = Pair(WakeupTime(null, null), true)
        deactivationFunction.invoke(wakeupClient, key)
    }

    fun expireCache(key: T) {
        wakeupTimeMap.remove(key)
    }

}