package eu.pozimski.alarmclockconfigurator.ui.elements

import android.os.Build
import androidx.annotation.RequiresApi
import androidx.compose.foundation.*
import androidx.compose.foundation.layout.*
import androidx.compose.material.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.ExperimentalUnitApi
import androidx.compose.ui.unit.TextUnit
import androidx.compose.ui.unit.TextUnitType
import androidx.compose.ui.unit.dp
import com.google.accompanist.swiperefresh.SwipeRefresh
import com.google.accompanist.swiperefresh.rememberSwipeRefreshState
import eu.pozimski.alarmclockconfigurator.http.WakeupTime
import eu.pozimski.alarmclockconfigurator.http.WakeupTimeCache
import eu.pozimski.alarmclockconfigurator.viewmodels.WakeupTimeViewModel
import io.ktor.util.collections.*
import kotlinx.datetime.DayOfWeek
import java.time.format.TextStyle
import java.util.*

@RequiresApi(Build.VERSION_CODES.TIRAMISU)
@Composable
fun WeekDayView(weekDayCache: WakeupTimeCache<java.time.DayOfWeek>) {
    val dayOfWeekViewModels: MutableMap<DayOfWeek, WakeupTimeViewModel<DayOfWeek>> =
        remember {
            ConcurrentMap()
        }

    var refreshing by remember {
        mutableStateOf(false)
    }

    SwipeRefresh(state = rememberSwipeRefreshState(refreshing), onRefresh = {
        refreshing = true
        dayOfWeekViewModels.values.forEach { viewModel -> viewModel.refresh() }
        refreshing = false
    }) {
        Column(
            modifier = Modifier
                .verticalScroll(rememberScrollState())
                .fillMaxSize()
        ) {
            WeekDayColumn(
                DayOfWeek.MONDAY,
                retrieveFromMap(dayOfWeekViewModels, weekDayCache, DayOfWeek.MONDAY)
            )
            WeekDayColumn(
                DayOfWeek.TUESDAY,
                retrieveFromMap(dayOfWeekViewModels, weekDayCache, DayOfWeek.TUESDAY)
            )
            WeekDayColumn(
                DayOfWeek.WEDNESDAY,
                retrieveFromMap(dayOfWeekViewModels, weekDayCache, DayOfWeek.WEDNESDAY)
            )
            WeekDayColumn(
                DayOfWeek.THURSDAY,
                retrieveFromMap(dayOfWeekViewModels, weekDayCache, DayOfWeek.THURSDAY)
            )
            WeekDayColumn(
                DayOfWeek.FRIDAY,
                retrieveFromMap(dayOfWeekViewModels, weekDayCache, DayOfWeek.FRIDAY)
            )
            WeekDayColumn(
                DayOfWeek.SATURDAY,
                retrieveFromMap(dayOfWeekViewModels, weekDayCache, DayOfWeek.SATURDAY)
            )
            WeekDayColumn(
                DayOfWeek.SUNDAY,
                retrieveFromMap(dayOfWeekViewModels, weekDayCache, DayOfWeek.SUNDAY)
            )
        }
    }
}

@OptIn(ExperimentalUnitApi::class)
@RequiresApi(Build.VERSION_CODES.TIRAMISU)
@Composable
private fun WeekDayColumn(
    weekDay: DayOfWeek,
    wakeupTimeViewModel: WakeupTimeViewModel<DayOfWeek>
) {
    Column {
        Row {
            Column(Modifier.fillMaxWidth(0.40F)) {
                Text(
                    text = weekDay.getDisplayName(TextStyle.FULL, Locale.ENGLISH) + ":",
                    fontSize = TextUnit(
                        25F, TextUnitType.Sp
                    )
                )
            }
            Column {
                WakeupTimeTextField(weekDay, wakeupTimeViewModel)
            }

        }
        Spacer(modifier = Modifier.height(5.dp))
    }
}

@RequiresApi(Build.VERSION_CODES.TIRAMISU)
@OptIn(ExperimentalFoundationApi::class, ExperimentalUnitApi::class)
@Composable
fun WakeupTimeTextField(dayOfWeek: DayOfWeek, viewModel: WakeupTimeViewModel<DayOfWeek>) {

    var dropDownExpanded by remember { mutableStateOf(false) }
    var showDeleteAlertDialog by remember { mutableStateOf(false) }
    var showTimePickerDialog by remember {
        mutableStateOf(false)
    }
    val backGroundColor by viewModel.backGroundColor.collectAsState()
    var currentError by remember {
        mutableStateOf<String?>(null)
    }

    Card(
        elevation = 4.dp,
        border = BorderStroke(2.dp, Color.Gray),
        backgroundColor = backGroundColor
    ) {
        Box(
            contentAlignment = Alignment.Center, modifier = Modifier
                .padding(5.dp)
                .background(backGroundColor)
        ) {
            Text(
                text = viewModel.clockTime,
                modifier = Modifier
                    .combinedClickable(onClick = { showTimePickerDialog = true },
                        onLongClick = {
                            dropDownExpanded = true
                        }),
                fontSize = TextUnit(25F, TextUnitType.Sp)
            )
        }
    }

    if (showTimePickerDialog) {
        TimePicker(
            hour = viewModel.hour,
            minute = viewModel.minute,
            onTimeSetListener = { _, newHour: Int, newMinute: Int ->
                currentError = viewModel.setWakeupTime(WakeupTime(newHour, newMinute))
                showTimePickerDialog = false
            },
            onDismissListener = { showTimePickerDialog = false })
    }

    DropdownMenu(expanded = dropDownExpanded, onDismissRequest = { dropDownExpanded = false }) {
        DropdownMenuItem(onClick = { showDeleteAlertDialog = true }) {
            Text("Delete")
        }
    }

    if (showDeleteAlertDialog) {
        DeleteAlertDialog(onDismissRequest = {
            showDeleteAlertDialog = false
            dropDownExpanded = false
        }, onConfirm = {
            showDeleteAlertDialog = false
            dropDownExpanded = false
            currentError = viewModel.deleteWakeupTime()
        },
            deletionElementNameCreator = {
                dayOfWeek.getDisplayName(
                    TextStyle.FULL,
                    Locale.ENGLISH
                )
            })
    }

    if (currentError != null) {
        ErrorDialog(onDismissRequest = { currentError = null }, currentError = currentError)
    }
}

fun retrieveFromMap(
    map: MutableMap<DayOfWeek, WakeupTimeViewModel<DayOfWeek>>,
    cache: WakeupTimeCache<DayOfWeek>,
    key: DayOfWeek
): WakeupTimeViewModel<DayOfWeek> {
    return map.computeIfAbsent(key) { WakeupTimeViewModel(cache, key) }
}