package eu.pozimski.alarmclockconfigurator.viewmodels

import android.util.Log
import androidx.compose.ui.graphics.Color
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import eu.pozimski.alarmclockconfigurator.http.WakeupTimeCache
import eu.pozimski.alarmclockconfigurator.http.WakeupTime
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking
import java.util.concurrent.atomic.AtomicReference

class WakeupTimeViewModel<T>(
    private val wakeupTimeCache: WakeupTimeCache<T>,
    private val date: T
) : ViewModel() {
    private var _wakeupTime = AtomicReference<WakeupTime?>(null)
    private val _backGroundColor = MutableStateFlow(Color.LightGray)
    val backGroundColor: StateFlow<Color> get() = _backGroundColor

    init {
        viewModelScope.launch {
            try {
                _wakeupTime.set(wakeupTimeCache.get(date))
                updateBackGroundColor()
            } catch (e: Exception) {
                e.message?.let { Log.println(Log.ERROR, "", it) }
            }
        }
    }

    private val wakeupTime: WakeupTime?
        get() = _wakeupTime.get()

    fun setWakeupTime(wakeupTime: WakeupTime): String? {
        try {
            runBlocking { wakeupTimeCache.put(date, wakeupTime) }
        } catch (e: Exception) {
            return e.message
        }
        _wakeupTime.set(wakeupTime)
        updateBackGroundColor()
        return null
    }

    fun deleteWakeupTime(): String? {
        try {
            runBlocking {
                wakeupTimeCache.delete(date)
            }
        } catch (e: Exception) {
            return e.message
        }
        _wakeupTime.set(null)
        updateBackGroundColor()
        return null;
    }

    fun deactivateWakeupTime(): String? {
        try {
            runBlocking {
                wakeupTimeCache.deactivate(date)
            }
        } catch (e: Exception) {
            return e.message
        }

        _wakeupTime.set(WakeupTime(null, null))
        updateBackGroundColor()
        return null
    }

    private fun updateBackGroundColor() {
        if (wakeupTime == null) {
            _backGroundColor.value = Color.White
        } else if (wakeupTime!!.minute == null && wakeupTime!!.hour == null) {
            _backGroundColor.value = Color.Red
        } else {
            _backGroundColor.value = Color.Green
        }
    }

    val minute: Int
        get() = if (wakeupTime != null && wakeupTime!!.minute != null) (
                wakeupTime!!.minute!!
                ) else {
            0
        }

    val hour: Int
        get() = if (wakeupTime != null && wakeupTime!!.hour != null) (
                wakeupTime!!.hour!!
                ) else {
            0
        }

    val clockTime: String
        get() = if (wakeupTime != null && wakeupTime!!.hour != null && wakeupTime!!.minute != null) {
            String.format("%02d:%02d", wakeupTime!!.hour, wakeupTime!!.minute)
        } else {
            "00:00"
        }

    fun refresh() {
        viewModelScope.launch {
            _backGroundColor.emit(Color.LightGray)
            wakeupTimeCache.expireCache(date)
            _wakeupTime.set(wakeupTimeCache.get(date))
            updateBackGroundColor()
        }
    }
}